﻿// Copyright 2012 rubicon informationstechnologie gmbh
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using Microsoft.FxCop.Sdk;
using NUnit.Framework;

namespace InjectionCop.IntegrationTests.Parser.TypeParsing.TypeParserTests.Return
{
  [TestFixture]
  public class Return_TypeParserTest : TypeParserTestBase
  {
    [Test]
    public void Parse_SafeSource_NoProblem()
    {
      Method sample = TestHelper.GetSample<ParserSampleBase>("SafeSource");
      _typeParser.Parse (sample);
      ProblemCollection result = _typeParser.Problems;

      Assert.That (TestHelper.ContainsProblemID (c_InjectionCopRuleId, result), Is.False);
    }

    [Test]
    public void Parse_UnsafeSource_ReturnsProblem()
    {
      Method sample = TestHelper.GetSample<ReturnSample>("ReturnFragmentMismatch");
      _typeParser.Parse (sample);
      ProblemCollection result = _typeParser.Problems;

      Assert.That (TestHelper.ContainsProblemID (c_InjectionCopRuleId, result), Is.True);
    }

    [Test]
    public void Parse_NoReturnAnnotation_NoProblem()
    {
      Method sample = TestHelper.GetSample<ReturnSample>("NoReturnAnnotation");
      _typeParser.Parse (sample);
      ProblemCollection result = _typeParser.Problems;

      Assert.That (TestHelper.ContainsProblemID (c_InjectionCopRuleId, result), Is.False);
    }

    [Test]
    public void Parse_DeclarationWithReturn_NoProblem ()
    {
      Method sample = TestHelper.GetSample<ReturnSample> ("DeclarationWithReturn");
      _typeParser.Parse (sample);
      ProblemCollection result = _typeParser.Problems;

      Assert.That (TestHelper.ContainsProblemID (c_InjectionCopRuleId, result), Is.False);
    }

    [Test]
    public void Parse_ReturnsDummyType_NoProblem ()
    {
      Method sample = TestHelper.GetSample<ReturnSample> ("ReturnsDummyType");
      _typeParser.Parse (sample);
      ProblemCollection result = _typeParser.Problems;

      Assert.That (TestHelper.ContainsProblemID (c_InjectionCopRuleId, result), Is.False);
    }
    
    [Test]
    public void Parse_ReturnsFieldWithWrongType_ReturnsProblem()
    {
      Method sample = TestHelper.GetSample<ReturnSample>("ReturnsFieldWithWrongType");
      _typeParser.Parse (sample);
      ProblemCollection result = _typeParser.Problems;

      Assert.That (TestHelper.ContainsProblemID (c_InjectionCopRuleId, result), Is.True);
    }
  }
}
